package signtools;

import java.io.File;

import java.net.URL;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

public class LaunchEnvelopingSignFile {
    public LaunchEnvelopingSignFile() {
        super();
    }
    
    private boolean argsError = false;
    private List listErrorArgs = new ArrayList();
    private HashSet usedFlags = new HashSet();
    private Map options = new HashMap();

    private String fileToSign;
    private String keyStoreFileName;
    private String keyStorePassword;
    private String keyStoreAlias;
    private String outputFile;
    private String outputPath;
    
    private UseKeyStore uks;
    private SignFile signFile;

    private static Logger LOGGER = Logger.getLogger("signtools.LaunchEnvelopingSignFile");

    /**
     * Executable method.
     * @param args: Args must be a combination of parameter specification (flag, -something) and value (without -) see 
     * initUsedArgs for a list of parameters specification:
     *       (-k keystoreFile -p keystorePassword -os keystoreAlias -fts FileNameToSign -f OutputFileName)
     */
    public static void main(String[] args) {
        LaunchEnvelopingSignFile esf = new LaunchEnvelopingSignFile();
        if (esf.loadArgs(args)) {
            esf.launch();
        }
    }

    /**
     * Launch signature creation, using keyStoreFileName, keyStorePassword, keyStoreAlias, fileToSign and outpuFile
     * All these values are usually loaded with method loadArgs.
     */
    private void launch() {
        try {
            setUks(new UseKeyStore());
            getUks().loadJavaKeyStore(getKeyStoreFileName(), getKeyStorePassword().toCharArray());
            //uks.setKeyStoreAlias(keyStoreAlias);
            getUks().getKeys(getKeyStoreAlias());
            if (getUks().getKey() == null) {
                LOGGER.warning("Error: " + "Can't find private key. JKS file: " + getKeyStoreFileName() +
                               ", password " + getKeyStorePassword() + ", alias: " + getKeyStoreAlias());
            } else if (getUks().getCer() == null) {
                LOGGER.warning("Error: " + "Can't find Public Key/certificate. JKS file: " + getKeyStoreFileName() +
                               ", password " + getKeyStorePassword() + ", alias: " + getKeyStoreAlias());
            } else {
                setSignFile(new SignFile());
                getSignFile().generateEnvelopingSignature(fileToSign, getOutputFile(), (PrivateKey)getUks().getKey(),
                                                     (X509Certificate)getUks().getCer());
            }
        } catch (Exception e) {
            //System.out.println("Exception at EnvelopingSignFile->launch: " + e.getMessage());
            LOGGER.warning("Exception " + e.getMessage());
        }
        //System.out.println("");
    }

    /**
     * Load and initilize arguments.
     * @param args: Arguments, same as in main.
     * @return: true if everything is ok. false if there is any problem. (note, true/false as boolean primitive, not Boolean object).
     */
    private boolean loadArgs(String[] args) {
        initUsedArgs();
        boolean resFlag = false;
        parse(args);
        String resStr = checkArgs();
        if (resStr.length() == 0) {
            //System.out.println("Args looks ok, let see if they are usefull");
            resFlag = true;
        } else {
            LOGGER.warning("Syntax error. Use with params:\n" +
                    resStr);
            resFlag = false;
        }
        return resFlag;
    }

    /**
     * Initializes usedFlag with a list of allowed flags (parameter specifications)
     */
    private void initUsedArgs() {
        getUsedFlags().add("-k"); //Java KeyStore file
        getUsedFlags().add("-p"); //Java KeyStore password
        getUsedFlags().add("-os"); //Java KeyStore alias
        //usedFlags.add("-d"); //FilePath of data instances. Used as URI
        getUsedFlags().add("-fts"); //File/URL to sign. Full file path
        //usedFlags.add("-o"); //Output directory?
        getUsedFlags().add("-f"); //Output File/URL? (why different from output directory?)
    }

    /**
     *Do whatever checking on the arguments
     * @return: String with the error/s (if any)
     */
    private String checkArgs() {
        String strRes = "";
        String param, message;
        if (getOptions() == null || getOptions().isEmpty()) {
            message =
                    "Use with parameters: " + "-k keystoreFile " + "-p keystorePassword " + "-os keystoreAlias " + "-fts FileNameToSign " +
                    "-f OutputFileName";
            strRes = strRes + message;
        } else {
            param = "-k";
            message = "Missing parameter -k. Use: -k keystoreFile\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                setKeyStoreFileName(((String)getOptions().get(param)).trim());
            }
            param = "-p";
            message = "Missing parameter -p. Use: -p keystorePassword\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                setKeyStorePassword(((String)getOptions().get(param)).trim());
            }
            param = "-os";
            message = "Missing parameter -os. Use: -os keystoreAlias\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                setKeyStoreAlias(((String)getOptions().get(param)).trim());
            }
            /*
            param = "-d";
            message = "Missing parameter -d. Use: -d PathFileToSign\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                URI = ((String)options.get(param)).trim();
            }
            */
            param = "-fts";
            message = "Missing parameter -fts. Use: -fts FileNameToSign\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                fileToSign = ((String)getOptions().get(param)).trim();
                try {
                    File f = new File(fileToSign);
                    if ((f.toString()).trim().length() <= 0) {
                        strRes = strRes + message;
                    }
                } catch (Exception ex) {
                    strRes = strRes + message;
                    fileToSign = "";
                }
            }
            /*param = "-o";
            message = "Missing parameter -o. Use: -o OutputDirectory\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                outputPath = ((String)options.get(param)).trim();
            }*/
            param = "-f";
            message = "Missing or bad parameter -f. Use: \"-f OutputFileName\" Note OutputFileName can be a URL.\n";
            if (!getOptions().containsKey(param) || ((String)getOptions().get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                setOutputFile(((String)getOptions().get(param)).trim());
                try {
                    File f = new File(getOutputFile());
                    /*f.mkdirs();
                    f.delete();
                    if ((f.toURI().toURL().toString()).trim().length()<=0) {
                        strRes = strRes + message;
                    }
                    else{
                        outputFile=f.toURI().toURL().toString().trim();
                    }*/
                } catch (Exception ex) {
                    strRes = strRes + message;
                    setOutputFile("");
                }
            }
        }

        return strRes;
    }

    /**
     * Parse parameters from arguments to its corresponding place (inside options)
     * @param args: Same as in the main method
     * @return: true if everything is ok. false otherwise.
     */
    private boolean parse(String[] args) {
        for (int n = 0; n < args.length; ++n) {
            if (args[n].charAt(0) == '-') {
                String name = args[n];
                String value = null;

                if (getUsedFlags().contains(args[n]) && n < args.length - 1){
                    value = args[++n];
                    getOptions().put(name, value);
                }
            }

            else {
                getListErrorArgs().add(args[n]);
                setArgsError(true);
            }
        }
        return isArgsError();
    }

    public boolean isArgsError() {
        return argsError;
    }

    public void setArgsError(boolean argsError) {
        this.argsError = argsError;
    }

    public List getListErrorArgs() {
        return listErrorArgs;
    }

    public void setListErrorArgs(List listErrorArgs) {
        this.listErrorArgs = listErrorArgs;
    }

    public HashSet getUsedFlags() {
        return usedFlags;
    }

    public void setUsedFlags(HashSet usedFlags) {
        this.usedFlags = usedFlags;
    }

    public Map getOptions() {
        return options;
    }

    public void setOptions(Map options) {
        this.options = options;
    }

    public String getKeyStoreFileName() {
        return keyStoreFileName;
    }

    public void setKeyStoreFileName(String keyStoreFileName) {
        this.keyStoreFileName = keyStoreFileName;
    }

    public String getKeyStorePassword() {
        return keyStorePassword;
    }

    public void setKeyStorePassword(String keyStorePassword) {
        this.keyStorePassword = keyStorePassword;
    }

    public String getKeyStoreAlias() {
        return keyStoreAlias;
    }

    public void setKeyStoreAlias(String keyStoreAlias) {
        this.keyStoreAlias = keyStoreAlias;
    }

    public String getOutputFile() {
        return outputFile;
    }

    public void setOutputFile(String outputFile) {
        this.outputFile = outputFile;
    }

    public String getOutputPath() {
        return outputPath;
    }

    public void setOutputPath(String outputPath) {
        this.outputPath = outputPath;
    }

    public UseKeyStore getUks() {
        return uks;
    }

    public void setUks(UseKeyStore uks) {
        this.uks = uks;
    }

    public SignFile getSignFile() {
        return signFile;
    }

    public void setSignFile(SignFile signFile) {
        this.signFile = signFile;
    }
}
